/**
 * WebSukui
 * copyright PowerBEANS Co., Ltd.
 * create date 2006/10/05
 * amonden
 */
package jp.co.powerbeans.common;

import java.util.MissingResourceException;

/**
 * Dict
 * @author amonden
 * @create date 2006/10/05
 */
public final class Dict {

  /**
   * 
   */
  private static final String ORDER = ".order";
  /**
   * 
   */
  private static final String PROP = "dict";
  private static java.util.ResourceBundle bundle;
  static {
    try {
      bundle  = java.util.ResourceBundle.getBundle(PROP);
    } catch (java.util.MissingResourceException e) {
      System.err.println("[PBSWT] create \"" + PROP + ".properties\" in source root.");
    } catch (RuntimeException e) {
      e.printStackTrace();
    }
  }
  
  /**
   * name を翻訳する
   * @param name
   * @return 翻訳結果
   */
  public static String trans(String name) {
    if (!bundleCheck()) {
      return "";
    }
    try {
      return bundle.getString(name);
    } catch (MissingResourceException e) {
      System.err.println("[PBSWT] can't translate from " + name + ". please add value to " + PROP + ".properties (ex: " + name + "=DisplayLabel");
      return name;
    } catch (RuntimeException e) {
      throw e;
    }
  }

  
  /**
   * 存在するはずの項目数 を指定して name を翻訳する
   * @param name
   * @param itemnum 項目数
   * @return 翻訳結果
   */
  public static String[] transMulti(String name, String[] default_vals) {
    int itemnum = default_vals.length;
    if (!bundleCheck()) {
      return default_vals;
    }
    try {
      String[] items = bundle.getString(name).split(",");
      if (items.length != itemnum) {
        System.err.println("[PBSWT] can't match item num " + name + ". please add value to " + PROP + ".properties (ex: " + name + "=label1,label2,..");
        return default_vals;
      }
      return items;
    } catch (MissingResourceException e) {
      System.err.println("[PBSWT] can't translate from " + name + ". please add value to " + PROP + ".properties (ex: " + name + "=DisplayLabel");
      return default_vals;
    } catch (RuntimeException e) {
      throw e;
    }
  }
  /**
   * 
   */
  private static boolean bundleCheck() {
    if (bundle == null) {
      System.err.println("[PBSWT] can't create resouce. please create \"" + PROP + ".properties\" in source root.");
      return false;
    }
    return true;
  }

  /**
   * @param model_name
   * @param name1
   * @param name2
   * @return
   */
  public static int compare(String model_name, String simple_model_name, String name1, String name2) {
    String order = "";
    String key = "";
    try {
      key = model_name + ORDER;
      order = bundle.getString(key);
    } catch (RuntimeException e) {
      // 存在しなくてもいいので無視
      try {
        key = simple_model_name + ORDER;
        order = bundle.getString(key);
      } catch (RuntimeException e1) {
        System.err.println("[PBSWT] can't get order of " + key + ". please order value to " + PROP + ".properties (ex: " + key + "=name,old");
      }
    }
    
    // 各プロパティのインデックスを取得
    int name1ind = order.indexOf(name1);
    int name2ind = order.indexOf(name2);
    // 両方とも -1 の場合は判別不能
    if (name1ind == -1 && name2ind == -1) {
      System.err.println("[PBSWT] can't get order of " + key + "." + name1 + "," + name2 + " . please order value to " + PROP + ".properties (ex: " + key + "=name,old");
    } else if (name1ind == -1) {
      return 1;
    } else if (name2ind == -1) {
      return -1;
    }
    return name1ind > name2ind ? 1 : -1;
  }


  /**
   * @param key
   * @return
   */
  public static boolean hasKey(String key) {
    try {
      bundle.getString(key);
      return true;
    } catch (RuntimeException e) {
      return false;
    }
  }

}
