/**
 * WebSukui
 * create date 2006/10/02
 * amonden
 */
package jp.co.powerbeans.websukui.logic;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.net.URL;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Set;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 * WebRoundMgr
 * 
 * @author amonden
 * @create date 2006/10/02
 */
public class WebRoundMgr {

	private String startUrl;

	private String ptn;

	private WebRoundMatchListener matchListener;

	/**
	 * 正規表現フラグ
	 */
	private boolean regmatch;

	// URLパターン1(完全URL)
	private static final Pattern URLP = Pattern
			.compile("(https?://[-_.!~*'a-zA-Z0-9;/\\?:\\@&=+\\$,%#]+html?)");

	// URLパターン2(サブURL)
	private static final Pattern URLP_SUB = Pattern
			.compile("(href|HREF)=\"([-_.!~*'a-zA-Z0-9;/\\?:\\@&=+\\$,%#]+html?)\"");
	/**
	 * @param text
	 * @param string
	 */
	public WebRoundMgr(String start_url, String ptn,
			WebRoundMatchListener match_listener, boolean regmatch) {
		this.startUrl = start_url;
		this.ptn = ptn;
		this.matchListener = match_listener;
		this.regmatch = regmatch;
	}

	// /* (non-Javadoc)
	// * @see java.lang.Thread#run()
	// */
	// @Override
	// public void run() {
	// super.run();
	//
	// String[] next_urls = parse(this.startUrl, this.ptn);
	//
	// // ここでどうするか？
	//
	// }

	public void start(int depth) {
		Set rounded_url = new HashSet();
		parse(this.startUrl, this.ptn, depth, rounded_url);
		this.matchListener.onFinish();
	}

	/**
	 * @param root_url
	 * @param ptn
	 * @param depth
	 * @param rounded_url
	 * @return
	 */
	private String[] parse(String root_url, String ptn, int depth,
			Set rounded_url) {

		if (rounded_url.contains(root_url)) {
			// 巡回済み
			return new String[] {};
		}
		rounded_url.add(root_url);

		this.matchListener.onParseStartUrl(root_url, depth);

		Set urllist = new HashSet();
		List result_list = new ArrayList();
		try {

			// URLクラスのインスタンスを生成
			URL url = new URL(root_url);
			// 入力ストリームを生成
			BufferedReader in = new BufferedReader(new InputStreamReader(url
					.openStream()));

			// 検索パターン
			Pattern p = this.regmatch ? Pattern.compile(ptn) : null;

			// 一行ずつ読み込みます
			String line;
			String charset = null;
			while ((line = in.readLine()) != null) {
				// URL 検索
				findUrl(URLP, line, urllist, "");
				findUrl(URLP_SUB, line, urllist, root_url);

				if (charset == null) {
					// 文字コード取得
					int top = -1;
					if ( (top = line.toUpperCase().indexOf("CHARSET=")) != -1) {
						top += 8;
            try {
              String lines = line.replace('\'', '"');
              charset = lines.substring(top, lines.indexOf("\"", top));
            } catch(java.lang.StringIndexOutOfBoundsException e) {
              continue;
            }
					}
				} else {
					// charset に変換
					line = new String(line.getBytes(), charset);
				}
				
				// ターゲット検索
				if (p == null) {
					// 通常検索し一致した文字列の前後を制御,記号文字まで取得
					int left = line.indexOf(ptn);
					if (left != -1) {
						result_list.add(ptn);
					}
					
				} else {
					// 正規表現
					Matcher m = p.matcher(line);
					if (m.find()) {
						for (int i = 0; i < m.groupCount(); i++) {
							result_list.add(m.group(i));
						}
					}
				}
			}

			// 入力ストリームを閉じます
			in.close();

			// 通知
			this.matchListener.onMatch(root_url, (String[]) result_list
					.toArray(new String[] {}));

			// 次のURLを取得
			if (depth > 1) {
				for (Iterator it = urllist.iterator(); it.hasNext();) {
					parse((String) it.next(), ptn, depth - 1, rounded_url);
				}
			}

		} catch (IOException e) {
			// e.printStackTrace();
		}
		return (String[]) urllist.toArray(new String[] {});
	}

	/**
	 * @param urlp
	 * @param line
	 * @param rounded_url
	 * @param urllist
	 * @param root_url
	 */
	private void findUrl(Pattern urlp, String line, Set urllist, String root_url) {
		Matcher urlm = urlp.matcher(line);
		if (urlm.find()) {
			for (int i = urlm.groupCount(); i <= urlm.groupCount(); i++) {
				String addurl = urlm.group(i);
				// href= ," がついている場合は削除
				addurl = addurl.toLowerCase().replaceAll("href=", "")
						.replaceAll("\"", "");

				// 完全URLか相対パスか判定
				if (addurl.startsWith("http")) {
					// 完全なのでそのまま
				} else if (addurl.startsWith("/")) {
					// ドメイン直下の相対パスなのでドメインを追加
					int sl = 0;
					for (int j = 0; j < 3 && sl != -1; j++) {
						sl = root_url.indexOf('/', sl + 1);
					}
					addurl = sl == -1 ? root_url + "/" + addurl : root_url
							.substring(0, sl)
							+ addurl;
				} else {
					// 相対パス
					addurl = root_url.substring(0, root_url.lastIndexOf('/'))
							+ "/" + addurl;
				}

				if (!urllist.contains(addurl)) {
					urllist.add(addurl);
				}
			}
		}
	}
}
