/**
 * WebSukui
 * copyright PowerBEANS Co., Ltd.
 * create date 2006/10/05
 * amonden
 */
package jp.co.powerbeans.websukui.swt;

import java.beans.BeanInfo;
import java.beans.IntrospectionException;
import java.beans.Introspector;
import java.beans.PropertyDescriptor;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import org.hsqldb.resources.BundleHandler;

import jp.co.powerbeans.common.Dict;

/**
 * PBActiveModelUtil
 * @author amonden
 * @create date 2006/10/05
 */
public class DictUtil {

  private static final String MODEL_SUFFIX_LIST_HIDDEN = "LH";
  private static final String MODEL_SUFFIX_EDIT_HIDDEN = "EH";
  private static final String MODEL_SUFFIX_BOOL_LABELS = ".LABELS";
  private static final String[] DEF_BOOL_LABELS = {"No","Yes"};
  private static final String IDNAME = ".IdName";

  /**
   * @param master_cls
   * @return
   */
  public static String[] getListDisplayColumns(Class master_cls) {
    List cols = new ArrayList();
    String[] colary = {};
    try {
      BeanInfo info = Introspector.getBeanInfo(master_cls);
      PropertyDescriptor[] ps = info.getPropertyDescriptors();
      for (int i = 0; i < ps.length; i++) {
        PropertyDescriptor p = ps[i];
        if (p.getWriteMethod() != null && !p.getName().toUpperCase().endsWith(MODEL_SUFFIX_LIST_HIDDEN)) {
//          cols.add(DynaResouce.trans(p.getName()));
          cols.add(p.getName());
        }
      }
      
      Collections.sort(cols, new ColumnCompare(master_cls));
      colary = (String[]) cols.toArray(new String[] {});
      for (int i = 0; i < colary.length; i++) {
        colary[i] = Dict.trans(colary[i]);
      } 

    } catch (IntrospectionException e) {
      e.printStackTrace();
    }
    return colary;
  }

  /**
   * @param master_cls
   * @return
   */
  public static String[] getListDisplayRowColumns(Class master_cls) {
    List cols = new ArrayList();
    String[] colary = {};
    try {
      BeanInfo info = Introspector.getBeanInfo(master_cls);
      PropertyDescriptor[] ps = info.getPropertyDescriptors();
      for (int i = 0; i < ps.length; i++) {
        PropertyDescriptor p = ps[i];
        if (p.getWriteMethod() != null && !p.getName().toUpperCase().endsWith(MODEL_SUFFIX_LIST_HIDDEN)) {
//          cols.add(DynaResouce.trans(p.getName()));
          cols.add(p.getName());
        }
      }
      
      Collections.sort(cols, new ColumnCompare(master_cls));
      colary = (String[]) cols.toArray(new String[] {});

    } catch (IntrospectionException e) {
      e.printStackTrace();
    }
    return colary;
  }
  /**
   * @param masterCls
   */
  public static String[] getEditDisplayRowColumns(Class master_cls) {
    List cols = new ArrayList();
    String[] colary = {};
    try {
      BeanInfo info = Introspector.getBeanInfo(master_cls);
      PropertyDescriptor[] ps = info.getPropertyDescriptors();
      for (int i = 0; i < ps.length; i++) {
        PropertyDescriptor p = ps[i];
        if (p.getWriteMethod() != null && !p.getName().toUpperCase().endsWith(MODEL_SUFFIX_EDIT_HIDDEN)) {
//          cols.add(DynaResouce.trans(p.getName()));
          cols.add(p.getName());
        }
      }
      
      Collections.sort(cols, new ColumnCompare(master_cls));
      colary = (String[]) cols.toArray(new String[] {});

    } catch (IntrospectionException e) {
      e.printStackTrace();
    }
    return colary;
  }

  /**
   * @param row_names
   * @return
   */
  public static String[] convertRowToDisp(String[] row_names) {
    String[] res = new String[row_names.length];

    for (int i = 0; i < res.length; i++) {
      res[i] = Dict.trans(row_names[i]);
    }
    return res;
  }

  /**
   * Booleanプロパティの表示ラベルを取得
   * @param cls モデルクラス
   * @param name プロパティ名
   * @return ラベル(String[2])
   */
  public static String[] getBooleanLabels(Class cls, String name) {
    return Dict.transMulti(cls.getSimpleName() + "." + name + MODEL_SUFFIX_BOOL_LABELS, DEF_BOOL_LABELS);
  }

  /**
   * @param class1
   * @param name
   * @param src_val
   * @return
   */
  public static String getBooleanValues(Class<? extends Object> cls, String name, String src_val) {
    String[] labels = getBooleanLabels(cls, name);
    int ind = ("0".equals(src_val) || "false".equals(src_val.toLowerCase()) || "no".equals(src_val.toLowerCase())) ? 
        0 : 1;
    return labels[ind];
  }

  /**
   * @param class1
   * @param name
   */
  public static boolean hasFKey(Class<? extends Object> cls, String name) {
    return name.endsWith("Id") && Dict.hasKey(cls.getSimpleName() + "." + name + IDNAME);
  }

  /**
   * @param cls
   * @param name
   * @return
   */
  public static Class getJoinClass(Class<? extends Object> cls, String name) {
    try {
      String ret_cls = getJoinVal(cls, name)[0];
      return Class.forName(ret_cls);
    } catch (ClassNotFoundException e) {
      System.err.println("can't get valid Class.property. please check value (ex: " + cls.getSimpleName() + "." + name + IDNAME + "=jp.co.powerbeans.CustomerGrp,groupName" );
      return null;
    }
  }
  /**
   * @param cls
   * @param name
   * @return
   */
  public static String getJoinColumn(Class<? extends Object> cls, String name) {
    return getJoinVal(cls, name)[1];
  }

  private static String[] getJoinVal(Class<? extends Object> cls, String name) {
    return Dict.trans(cls.getSimpleName() + "." + name + IDNAME).split(",");
  }

}
