/**
 * WebSukui
 * copyright PowerBEANS Co., Ltd.
 * create date 2006/10/10
 * amonden
 */
package jp.co.powerbeans.websukui.swt;

import java.io.UnsupportedEncodingException;
import java.util.Date;
import java.util.Properties;

import javax.mail.Message;
import javax.mail.MessagingException;
import javax.mail.Session;
import javax.mail.Transport;
import javax.mail.internet.InternetAddress;
import javax.mail.internet.MimeMessage;

import jp.co.powerbeans.websukui.model.Customer;
import jp.co.powerbeans.websukui.model.MailConfig;

/**
 * SendMailEnum
 * 
 * @author amonden
 * @create date 2006/10/10
 */
public enum SendMailConfirmType {
  TEST /* テスト送信モード */{
    public void send(MailConfig account, Customer[] customers, String test_to, String cc,
        String bcc, String subject, String body, SendMailStatusListener listener)
        throws UnsupportedEncodingException, MessagingException {

      transport(account, test_to, cc, bcc, subject, body);
    }

  },
  OFFICIAL /* 本番送信モード */{
    public void send(MailConfig account, Customer[] customers, String test_to, String cc,
        String bcc, String subject, String body, SendMailStatusListener listener)
        throws UnsupportedEncodingException, MessagingException {

      int all_num = customers.length;
      for (int i = 0; i < customers.length; i++) {
        transport(account, customers[i].getEmail(), cc, bcc, subject, body);
        listener.onSend(i + 1, all_num);
      }
    }
  };

  /**
   * メール送信を実行する
   * 
   * @param account
   * @param customers
   * @param test_to
   * @param cc
   * @param bcc
   * @param subject
   * @param body
   * @throws MessagingException
   * @throws UnsupportedEncodingException
   */
  public abstract void send(MailConfig account, Customer[] customers, String test_to, String cc,
      String bcc, String subject, String body, SendMailStatusListener listener)
      throws UnsupportedEncodingException, MessagingException;

  private static void transport(MailConfig account, String to, String cc, String bcc,
      String subject, String body) throws MessagingException, UnsupportedEncodingException {
    Properties props = System.getProperties();
    // SMTPサーバーのアドレスを指定
    props.put("mail.smtp.host", account.getMailSmtp());
    props.put("mail.host", account.getMailSmtp());
    props.put("mail.smtp.auth", "true");

    Session session = Session.getDefaultInstance(props, null);
    MimeMessage mimeMessage = new MimeMessage(session);

    // 送信元メールアドレスと送信者名を指定
    mimeMessage.setFrom(new InternetAddress(account.getMailEmail(), account.getMailName(),
        "iso-2022-jp"));
    // 送信先メールアドレスを指定
    mimeMessage.setRecipients(Message.RecipientType.TO, to);

    if (cc != null && cc.length() > 0) {
      mimeMessage.setRecipients(Message.RecipientType.CC, cc);
    }
    if (bcc != null && bcc.length() > 0) {
      mimeMessage.setRecipients(Message.RecipientType.BCC, bcc);
    }

    // メールのタイトルを指定
    mimeMessage.setSubject(subject, "ISO-2022-JP");

    mimeMessage.setText(body + "\n", "ISO-2022-JP");

    // メールの形式を指定
    mimeMessage.setHeader("Content-Type", "text/plain; charset=\"iso-2022-jp\"");
    mimeMessage.setHeader("Content-Transfer-Encoding", "7bit"); // PHS,携帯用文字化け対策

    // 送信日付を指定
    mimeMessage.setSentDate(new Date());

    // 送信します
    if (account.isMailUseSmtpAuth()) {
      // SMTP認証版
      javax.mail.Transport tp = session.getTransport("smtp");
      tp.connect(account.getMailSmtp(), account.getMailAccount(), account.getMailPasswordLH());
      InternetAddress[] tos = { new InternetAddress(to) };
      tp.sendMessage(mimeMessage, tos);

    } else {
      Transport.send(mimeMessage);
    }

  }
}
