package jp.co.powerbeans.websukui.swt;


import java.io.IOException;

import jp.co.powerbeans.common.db.DBManager;
import jp.co.powerbeans.common.db.DBUtil;
import jp.co.powerbeans.common.swt.MsgSwt;
import jp.co.powerbeans.websukui.Msg;
import jp.co.powerbeans.websukui.logic.WebRoundMatchListener;
import jp.co.powerbeans.websukui.logic.WebRoundMgr;
import jp.co.powerbeans.websukui.model.Config;
import jp.co.powerbeans.websukui.model.Customer;
import jp.co.powerbeans.websukui.model.CustomerGrp;

import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableItem;
import org.eclipse.swt.widgets.Text;

public class WebRoundSwt extends PBSwt implements WebRoundMatchListener {

  private static final String TITLE = Msg.getTitleMain();

  static final int WINDOW_WIDTH = 800;

  static final int WINDOW_HEIGTH = 600;

  static final int BUTTON_WIDTH = WINDOW_WIDTH;

  static final int BUTTON_HEIGTH = 50;

  private static final String[] DEF_PTNS =
  	{ "E-mail ([a-zA-Z0-9-_\\.]+@[a-zA-Z0-9-_\\.]+)",
      "URL (https?://[-_.!~*'a-zA-Z0-9;/\\?:\\@&=+\\$,%#]+html?)"};

  private static final String[] LABELS = { Msg.getMenuOutputHtml(), Msg.getMenuClear(),
      Msg.getMenuImportEmail() };

  private static final String[] CMDS = { "export", "clear", "importEmail" };

  private static final String[] DEPTHS = { "1", "2", "3", "4", "5", "6", "7", "8", "9" };

  private static final String[] TIMES = { "1", "2", "3", "4", "5", "6", "7", "8", "9" };

  private static final String[] IMPORT_EMAIL_PROPNAMES = { "", "url", "email" };

  private Combo targetPtnCombo;

  private String[] cols = { Msg.getColumnNo(), Msg.getColumnUrl(), Msg.getColumnTargetValue() };

  private int[] wids = { 50, 400, 200 };

  private Table logTable;

  private int rownum = 1;

  private Text logText;

  private Combo depthCombo;

  private Combo urlCombo;

  private Combo timesCombo;

  private Button duplicateDataDustCheck;

private Button regCheckBox;

  public WebRoundSwt() {
    super();
    initialize();
  }

  private void initialize() {
    createSShell(TITLE, WINDOW_WIDTH, WINDOW_HEIGTH, true);
    createItems();
    createMenu();
  }

  /**
   * 
   */
  private void createMenu() {
    create1LayerMenu(LABELS, CMDS);
  }

  /**
   * 
   */
  private void createItems() {
    setGridLayout(4);

    // 1行目 URL
    addLabel(Msg.getRoundUrl());
    this.urlCombo = addComboBoxEditable("urlCombo", new String[] {}, 2, 400);
    addSimpleButton_Left(Msg.getStartRound(), "start");

    // 2行目 収集正規表現選択コンボ
    addLabel(Msg.getTargetPtn());
    this.targetPtnCombo = addComboBoxEditable("targetPtnConbo", DEF_PTNS, 1, 300);
    this.targetPtnCombo.select(0);
    
    // 正規表現チェック
    this.regCheckBox = addCheckBox("正規表現");
    addLabel();

    // 3行目 回数指定コンボ
    addLabel(Msg.getRoundDepth());
    this.depthCombo = addComboBox("depthCombo", DEPTHS);
    this.depthCombo.select(0);
    
    this.duplicateDataDustCheck = addCheckBox(Msg.getDuplicateDataDust());
    addLabel();
    
    // 4行目
    addLabel(Msg.getRoundTimes());
    this.timesCombo = addComboBoxEditable("timesCombo", TIMES, 1, 100);
    this.timesCombo.select(0);
    addLabel();
    addLabel();

    // 5行目 収集結果テーブル
    this.logTable = addSimpleTable(cols, wids, 4);
    setCmdToTableOnSingleClick(this.logTable, "logTableClick");

    // 6行目 収集ログ
    this.logText = addTextBoxMultiReadOnly("logText", 4, 50);

    // 初期値格納
    setInitValues();

  }

  /**
   * 
   */
  private void setInitValues() {
    if (DBManager.countByAll(Config.class) == 0) {
      Config config = new Config();
      config.setConfigId(1);
      config.setRootUrl(Msg.getDefaultUrl());
      DBManager.insert(config);
    }

    Config[] configs = (Config[]) DBManager.findBy(Config.class, "", " configId DESC ");

    for (int i = 0; i < configs.length && i < 20; i++) {
      this.urlCombo.add(configs[i].getRootUrl());
    }
    if (this.urlCombo.getItemCount() > 0) {
      this.urlCombo.select(0);
    }
  }

  /**
   * テーブルクリック
   */
  public void cmdLogTableClick() {
    TableItem item = this.logTable.getItem(this.logTable.getSelectionIndex());
    String url = item.getText(1);

    // 開く
    try {
      Runtime.getRuntime().exec("explorer " + url);
    } catch (IOException e) {
      e.printStackTrace();
    }

  }

  /**
   * 巡回開始
   */
  public void cmdStart() {
    int times = 0;
    try {
      times = Integer.parseInt(this.timesCombo.getText());
    } catch (NumberFormatException e1) {
      MsgSwt.warn(Msg.getInvalidTimes());
      this.timesCombo.forceFocus();
      return;
    }

    try {
      for (int i = 0; i < times; i++) {
        String ptn = this.regCheckBox.getSelection() ? this.targetPtnCombo.getText().split(" ")[1] : this.targetPtnCombo.getText();
        // 巡回履歴を追加
        {
          Config config = new Config();
          config.setRootUrl(this.urlCombo.getText());
          if (DBManager.countBy(Config.class, " rootUrl = '" + config.getRootUrl() + "'") == 0) {
            DBManager.insert(config);
          }
          this.urlCombo.add(config.getRootUrl());
        }

        // 巡回開始
        WebRoundMgr mgr = new WebRoundMgr(this.urlCombo.getText(), ptn, this, this.regCheckBox.getSelection());
        mgr.start(Integer.parseInt(depthCombo.getText()));
      }

    } catch (RuntimeException e) {
      MsgSwt.warn(Msg.getInvalieTargetPtn());
      e.printStackTrace();
    }
  }

  public void cmdExit() {
    this.sShell.close();
  }

  public void cmdExport() {
    output1TableToHtml("result.html", "round result", this.logTable);
  }

  public void cmdClear() {
    this.logTable.removeAll();
    this.logText.setText("");
    this.rownum = 1;
  }

  /**
   * Emailデータとして取り込む
   */
  public void cmdImportEmail() {
    // グループ選択
    CustomerGrp m = new CustomerGrp();
    PBMasterSelectComboSwt swt = new PBMasterSelectComboSwt(m, "取り込むユーザーのグループを指定してください", "グループ", "groupName");
    
    if ("ok".equals(swt.show())) {
      // 登録
      Customer[] cs = (Customer[]) DBUtil.swtTableToModels(this.logTable, IMPORT_EMAIL_PROPNAMES,
          Customer.class);
      for (int i = 0; i < cs.length; i++) {
        cs[i].setCustomerGrpId(m.getCustomerGrpId());
        DBManager.insert(cs[i]);
      }

      MsgSwt.info(Msg.getRegisted());
    }
  }

  /*
   * (non-Javadoc)
   * 
   * @see jp.co.powerbeans.websukui.swt.WebRoundMatchListener#onMatch(java.lang.String,
   *      java.lang.String[])
   */
  public void onMatch(String url, String[] vals) {

    for (int i = 0; i < vals.length; i++) {
      if (this.duplicateDataDustCheck.getSelection()) {
        // 重複データは破棄. テーブルのデータをチェックして存在する場合は追加しない
        if (tableExist(this.logTable, 2, vals[0])) {
          continue;
        }
      }
      addTableRow(this.logTable, new String[] { "" + rownum++, url, vals[i] });
    }

    super.sShell.update();
    this.logTable.update();
  }

  /*
   * (non-Javadoc)
   * 
   * @see jp.co.powerbeans.websukui.swt.WebRoundMatchListener#onParseStartUrl(java.lang.String, int)
   */
  public void onParseStartUrl(String root_url, int depth) {
    String msg = "parse depth=" + depth + " url=" + root_url + "\r\n";
    this.logText.append(msg);
  }

  /*
   * (non-Javadoc)
   * 
   * @see jp.co.powerbeans.websukui.swt.WebRoundMatchListener#onFinish()
   */
  public void onFinish() {
    this.logText.append("end.\r\n");
  }
}
